# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.testing import TestRequest, tests
from zope.component import getMultiAdapter
from zope.interface.verify import verifyObject
from zeam.component import getWrapper

from silva.core.editor.transform.interfaces import ITransformerFactory
from silva.core.editor.transform.interfaces import ISaveEditorFilter
from silva.core.editor.transform.interfaces import IInputEditorFilter
from silva.core.editor.transform.interfaces import IDisplayFilter
from silva.core.views.interfaces import IPreviewLayer

from ..interfaces import IExternalSourceManager, IExternalSourceInstance
from ..testing import FunctionalLayer


class SourceTransformerTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('author')

        # Create a test document.
        factory = self.root.manage_addProduct['silva.app.document']
        factory.manage_addDocument('document', 'Document Example')


class CreateSourceTransformerTestCase(SourceTransformerTestCase):


    def transform(self, text, filter, version, public=True):
        """Helper to call transform.
        """
        layers = []
        if not public:
            layers = [IPreviewLayer]
        request = TestRequest(layers=layers)
        factory = getMultiAdapter((version, request), ITransformerFactory)
        transformer = factory('body', version.body, text, filter)
        return unicode(transformer)

    def test_display_broken_xml_source_public(self):
        """Test a source that have an invalid or missing
        data-source-instance.
        """
        version = self.root.document.get_editable()
        factory = getMultiAdapter(
            (version, TestRequest()), ITransformerFactory)
        transformer = factory(
            'body', version.body, '<div class="external-source"></div>',
            IDisplayFilter)

        tests.assertXMLEqual(
            unicode(transformer),
            """
<div class="external-source">
</div>
""")

    def test_display_broken_xml_source_preview(self):
        """Test a source that have an invalid or missing
        data-source-instance.
        """
        version = self.root.document.get_editable()
        factory = getMultiAdapter(
            (version, TestRequest(layers=[IPreviewLayer])),
            ITransformerFactory)
        transformer = factory(
            'body', version.body, '<div class="external-source"></div>',
            IDisplayFilter)

        tests.assertXMLEqual(
            unicode(transformer),
            """
<div class="external-source broken-source">
 <p>
   External Source parameters are missing.
 </p>
</div>
""")

    def test_create_source(self):
        """Create a source using the transformer.
        """
        version = self.root.document.get_editable()
        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 0)

        # We now save the source, this going to create it.
        saved_text = self.transform("""
<h1>
   Document Example
</h1>
<div class="external-source"
     data-silva-name="cs_citation"
     data-silva-settings="field_citation=Super%20citation&field_author=moi">
</div>
""", ISaveEditorFilter, version)

        # One source have been created.
        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 1)
        instance_key = list(sources.all())[0]
        parameters, source = sources.get_parameters(instance=instance_key)
        self.assertTrue(verifyObject(IExternalSourceInstance, parameters))
        self.assertEqual(parameters.get_source_identifier(), 'cs_citation')
        self.assertEqual(parameters.get_parameter_identifier(), instance_key)
        self.assertEqual(source.id, 'cs_citation')
        self.assertEqual(parameters.citation, u'Super citation')
        self.assertEqual(parameters.author, u'moi')
        self.assertEqual(parameters.source, u'')
        tests.assertXMLEqual(
            saved_text,
            """
<div class="external-source" data-source-instance="%s">
</div>
""" % instance_key)

        # You can render this source for the editor
        editor_text = self.transform(saved_text, IInputEditorFilter, version)
        tests.assertXMLEqual(
            editor_text, """
<h1>
   Document Example
</h1>
<div class="external-source" data-silva-instance="%s">
</div>
""" % instance_key)

        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 1)


class EditCustomSourceTransformerTestCase(SourceTransformerTestCase):

    def setUp(self):
        super(EditCustomSourceTransformerTestCase, self).setUp()
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addCodeSource('cs_source', 'Test Source', 'script')
        factory = self.root.cs_source.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script')
        script = self.root.cs_source._getOb('script')
        script.write("""
##parameters=model,version,REQUEST

return '<a href="http://silvacms.org">SilvaCMS</a><b>Silva is cool</b>'
""")
        version = self.root.document.get_editable()
        sources = getWrapper(version, IExternalSourceManager)
        controller = sources(TestRequest(form={}), name='cs_source')
        controller.create()
        self.instance_key = controller.getId()
        self.saved_text = """
<h1>This is a title</h1>
<div class="external-source" data-source-instance="%s">
</div>
<p>Well good bye!</p>
""" % self.instance_key

    def test_display_source_with_multi_tags(self):
        """Display and render a source that have multiple top level
        tags.
        """
        version = self.root.document.get_editable()
        factory = getMultiAdapter(
            (version, TestRequest()), ITransformerFactory)
        transformer = factory(
            'body', version.body, self.saved_text, IDisplayFilter)
        tests.assertXMLEqual(
            unicode(transformer),
"""
<h1>This is a title</h1>
<a href="http://silvacms.org">SilvaCMS</a>
<b>Silva is cool</b>
<p>Well good bye!</p>
""")
        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 1)


class EditSourceTransformerTestCase(SourceTransformerTestCase):

    def setUp(self):
        super(EditSourceTransformerTestCase, self).setUp()
        version = self.root.document.get_editable()
        sources = getWrapper(version, IExternalSourceManager)
        controller = sources(TestRequest(form={
                    'field_citation': "Don't trust citations on the Internet",
                    'field_author': "Charlemagne",
                    'field_source': "The Internet"}), name='cs_citation')
        controller.create()
        self.instance_key = controller.getId()
        self.saved_text = """
<div class="external-source" data-source-instance="%s">
</div>
""" % self.instance_key

    def test_display_source(self):
        """If you render without any layer, you will see the code
        source rendered.
        """
        version = self.root.document.get_editable()
        factory = getMultiAdapter(
            (version, TestRequest()), ITransformerFactory)
        transformer = factory(
            'body', version.body, self.saved_text, IDisplayFilter)
        tests.assertXMLEqual(
            unicode(transformer),
"""
<div class="citation">
 Don't trust citations on the Internet
 <div class="author">
  Charlemagne
 </div>
 <div class="source">
  The Internet
 </div>
</div>
""")
        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 1)

    def test_display_source_with_alignement(self):
        """If you render without any layer, you will see the code
        source rendered.
        """
        version = self.root.document.get_editable()
        factory = getMultiAdapter(
            (version, TestRequest()), ITransformerFactory)
        transformer = factory(
            'body', version.body, """
<div class="external-source float-left" data-source-instance="%s">
</div>
""" % self.instance_key, IDisplayFilter)
        tests.assertXMLEqual(
            unicode(transformer),
"""
<div class="external-source float-left">
 <div class="citation">
  Don't trust citations on the Internet
  <div class="author">
   Charlemagne
  </div>
  <div class="source">
   The Internet
  </div>
 </div>
</div>
""")
        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 1)

    def test_display_source_with_default_alignement(self):
        """If you render without any layer, you will see the code
        source rendered.
        """
        version = self.root.document.get_editable()
        factory = getMultiAdapter(
            (version, TestRequest()), ITransformerFactory)
        transformer = factory(
            'body', version.body, """
<div class="external-source default" data-source-instance="%s">
</div>
""" % self.instance_key, IDisplayFilter)
        tests.assertXMLEqual(
            unicode(transformer),
"""
<div class="citation">
 Don't trust citations on the Internet
 <div class="author">
  Charlemagne
 </div>
 <div class="source">
  The Internet
 </div>
</div>
""")
        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 1)

    def test_display_broken_source_preview(self):
        """If you delete the code source object, and render with a
        preview layer, you have an error message.
        """
        self.root.manage_delObjects(['cs_citation'])

        version = self.root.document.get_editable()
        factory = getMultiAdapter(
            (version, TestRequest(layers=[IPreviewLayer])),
            ITransformerFactory)
        transformer = factory(
            'body', version.body, self.saved_text, IDisplayFilter)
        tests.assertXMLEqual(
            unicode(transformer),
"""
<div class="external-source broken-source">
 <p>
   External Source unknow is not available.
 </p>
</div>
""")

        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 1)

    def test_display_broken_source_public(self):
        """If you delete the code source object, and render without a
        preview layer, you have no message.
        """
        self.root.manage_delObjects(['cs_citation'])

        version = self.root.document.get_editable()
        factory = getMultiAdapter(
            (version, TestRequest()),
            ITransformerFactory)
        transformer = factory(
            'body', version.body, self.saved_text, IDisplayFilter)
        tests.assertXMLEqual(
            unicode(transformer),
"""
<div class="external-source">
</div>
""")

        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 1)

    def test_delete_source(self):
        """If you save again without provided the html for the source,
        it is removed.
        """
        version = self.root.document.get_editable()
        factory = getMultiAdapter((version, TestRequest()), ITransformerFactory)
        transformer = factory('body', version.body, "", ISaveEditorFilter)
        tests.assertXMLEqual(
            unicode(transformer),
            "")

        sources = getWrapper(version, IExternalSourceManager)
        self.assertEqual(len(sources.all()), 0)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CreateSourceTransformerTestCase))
    suite.addTest(unittest.makeSuite(EditSourceTransformerTestCase))
    suite.addTest(unittest.makeSuite(EditCustomSourceTransformerTestCase))
    return suite




## Script (Python) "iframe_url_validator"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=value, REQUEST
##title=
##
if not value:
    return False

value = value.strip()

for possible_urls in context.allowed_urls:
    for url in possible_urls.split():
        if value.startswith(url):
            return True

return False



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from silva.core import conf as silvaconf

class CKEditorExtension(object):
    base = '++static++/Products.SilvaExternalSources.editor'
    plugins = {
        'silvaexternalsource': 'plugins/silvaexternalsource'
        }


class IEditorPluginResources(IDefaultBrowserLayer):
    silvaconf.resource('parameters.css')

extension = CKEditorExtension()



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import lxml.html
import re

from AccessControl import ModuleSecurityInfo

module_security = ModuleSecurityInfo(
    'Products.SilvaExternalSources.codesources.googlemaps')

# Google lists http://www.google.com/supported_domains but it does
# change regularly
_url_pattern = re.compile(
    r'^(https?://)?(www\.|maps\.)?google\.[a-z]{2,3}(\.[a-z]{2})?/(maps)?.*$')


# Validate that iFrame code pasted into the codesource has a valid googlemaps
# URL and that there are no nested iFrames inside.
# iFrames may not have any inline styles.

# The URLs for googlemaps iframes must either have (www.)?google.(TLD) for the
# domain and /maps for the beginning of the path, or have maps.google(TLD).
# for the domain. They must begin with https? schemes if there is a scheme.

module_security.declarePublic('validate_googlemaps_iframe')
def validate_googlemaps_iframe(iframe, REQUEST=None):
    """Validate the googlemaps iFrame HTML and URLs.

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="http://www.google.com/"></iframe><br /><small><a href="http://www.google.com/" style="color:#0000FF;text-align:left">View Larger Map</a></small>')
    True

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="http://www.google.com/"></iframe>')
    True

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="http://maps.google.com/skinnyelephants"></iframe><br /><small><a href="http://maps.google.com/skinnyelephantz" style="color:#0000FF;text-align:left">View Larger Map</a></small>')
    True

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="https://maps.google.com/maps?f=q&amp;ll=51.924216,4.481776&amp;output=embed"></iframe><br /><small><a href="https://maps.google.com/maps?f=q&amp;ll=51.924216,4.481776" style="color:#0000FF;text-align:left">View Larger Map</a></small>')
    True

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="maps.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782"></iframe><br /><small><a href="maps.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782" style="color:#0000FF;text-align:left">View Larger Map</a></small>')
    True

    >>> validate_googlemaps_iframe('<IFRAME STYLE=ZOMG! WIDTH=425 HEIGHT=350 FRAMEBORDER=0 SCROLLING=no MARGINHEIGHT=0 MARGINWIDTH=0 SRC=www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782></IFRAME><BR><SMALL><A HREF="www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782" STYLE="COLOR:#00F;TEXT-ALIGN:LEFT">View Larger Map</a></SMALL>')
    False

    >>> validate_googlemaps_iframe('<IFRAME WIDTH=425 HEIGHT=350 FRAMEBORDER=0 SCROLLING=no MARGINHEIGHT=0 MARGINWIDTH=0 SRC=www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782></IFRAME><BR><SMALL><A HREF="www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782" STYLE=COLOR:#00F;TEXT-ALIGN:LEFT>View Larger Map</a></SMALL>')
    True

    >>> validate_googlemaps_iframe('<IFRAME WIDTH=425 HEIGHT=350 FRAMEBORDER=0 SCROLLING=no MARGINHEIGHT=0 MARGINWIDTH=0 SRC=www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782></IFRAME><BR><P><SMALL><A HREF="www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782" STYLE="COLOR:#00F;TEXT-ALIGN:LEFT">View Larger Map</a></SMALL>')
    True

    >>> validate_googlemaps_iframe('''<IFRAME WIDTH=425 HEIGHT=350 FRAMEBORDER=0 SCROLLING=no MARGINHEIGHT=0 MARGINWIDTH=0 SRC=www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782></IFRAME><BR><P><SMALL><A HREF="Javascript:'content'" STYLE="COLOR:#00F;TEXT-ALIGN:LEFT">View Larger Map</a></SMALL>''')
    False

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="http://infrae.com"></iframe><br /><small><a href="http://infrae.com" style="color:#0000FF;text-align:left">View Larger Map</a></small>')
    False

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="https://maps.google.at/maps?f=q&amp;source=s_q&amp;hl=en&amp;geocode=&amp;q=Augasse+2-6,+Wien&amp;aq=0&amp;oq=augasse+2&amp;sll=47.696454,13.345766&amp;sspn=7.898759,22.115479&amp;t=h&amp;ie=UTF8&amp;hq=&amp;hnear=Augasse+2-6,+Alsergrund+1090+Wien&amp;z=14&amp;ll=48.231908,16.357079&amp;output=embed"></iframe><br /><small><a href="https://maps.google.at/maps?f=q&amp;source=embed&amp;hl=en&amp;geocode=&amp;q=Augasse+2-6,+Wien&amp;aq=0&amp;oq=augasse+2&amp;sll=47.696454,13.345766&amp;sspn=7.898759,22.115479&amp;t=h&amp;ie=UTF8&amp;hq=&amp;hnear=Augasse+2-6,+Alsergrund+1090+Wien&amp;z=14&amp;ll=48.231908,16.357079" style="color:#0000FF;text-align:left">View Larger Map</a></small>')
    True

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782"><iframe src="http://google.com.au/evilscript.pl"></iframe></iframe><br /><small><a href="www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782" style="color:#0000FF;text-align:left">View Larger Map</a></small>')
    False

    >>> validate_googlemaps_iframe('''<iframe script="javascript:alert('Playa')" width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782"><iframe src="http://google.com.au/evilscript.pl"></iframe></iframe><br /><small><a href="www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782" style="color:#0000FF;text-align:left">View Larger Map</a></small>''')
    False

    >>> validate_googlemaps_iframe('''<iframe style="color: red" width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782"><iframe src="http://google.com.au/evilscript.pl"></iframe></iframe><br /><small><a href="www.google.com/maps?saddr=Hotel+Ibis+Coimbra+%4040.205247,-8.426782" style="color:#0000FF;text-align:left">View Larger Map</a></small>''')
    False

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="http://www.google.com/"></iframe><iframe src="http://infrae.com"></iframe>')
    False

    >>> validate_googlemaps_iframe('<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="http://www.google.com/"></iframe><form action="http://infrae.com"><a>Form</a></form>')
    False

    >>> validate_googlemaps_iframe('''<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="https://maps.google.at/maps?f=q&amp;source=s_q&amp;hl=en&amp;geocode=&amp;q=Augasse+2-6,+Wien&amp;aq=0&amp;oq=augasse+2&amp;sll=47.696454,13.345766&amp;sspn=7.898759,22.115479&amp;t=h&amp;ie=UTF8&amp;hq=&amp;hnear=Augasse+2-6,+Alsergrund+1090+Wien&amp;z=14&amp;ll=48.231908,16.357079&amp;output=embed"></iframe><br /><small><a href="https://maps.google.at/maps?f=q&amp;source=s_q&amp;hl=en&amp;geocode=&amp;q=Augasse+2-6,+Wien&amp;aq=0&amp;oq=augasse+2&amp;sll=47.696454,13.345766&amp;sspn=7.898759,22.115479&amp;t=h&amp;ie=UTF8&amp;hq=&amp;hnear=Augasse+2-6,+Alsergrund+1090+Wien&amp;z=14&amp;ll=48.231908,16.357079&amp;output=embed" style="color:#0000FF;text-align:left">View Larger Map</a></small>''')
    True

    >>> validate_googlemaps_iframe('''<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="http://google.fa.ke/evil"></iframe><br /><small><a href="http://google.fa.ke/evil" style="color:#0000FF;text-align:left">View Larger Map</a></small>''')
    True

    >>> validate_googlemaps_iframe('''<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="https://maps.google.at/maps?f=q&amp;source=s_q&amp;hl=en&amp;geocode=&amp;q=Augasse+2-6,+Wien&amp;aq=0&amp;oq=augasse+2&amp;sll=47.696454,13.345766&amp;sspn=7.898759,22.115479&amp;t=h&amp;ie=UTF8&amp;hq=&amp;hnear=Augasse+2-6,+Alsergrund+1090+Wien&amp;z=14&amp;ll=48.231908,16.357079&amp;output=embed"></iframe><br /><small><a href="https://maps.google.at/maps?f=q&amp;source=embed&amp;hl=en&amp;geocode=&amp;q=Augasse+2-6,+Wien&amp;aq=0&amp;oq=augasse+2&amp;sll=47.696454,13.345766&amp;sspn=7.898759,22.115479&amp;t=h&amp;ie=UTF8&amp;hq=&amp;hnear=Augasse+2-6,+Alsergrund+1090+Wien&amp;z=14&amp;ll=48.231908,16.357079" style="color:#0000FF;text-align:left">View Larger Map</a></small>''')
    True
    """

    if iframe is None:
        return False

    tree = lxml.html.fragment_fromstring(iframe, create_parent=True)
    URLs = []
    elements = tree.findall('.//*')

    for element in elements:
        if element.tag == 'iframe':
            if (len(element.getchildren()) != 0 or
              'style' in element.keys() or
              'javascript' in element.values()):
                return False
        for key, value in element.items():
            #form actions, onclick, etc
            if (key.lower().startswith('on') or
              key.lower() == 'action' or 'javascript' in value.lower()):
                return False
            if key == 'src' or key == 'href':
               URLs.append(element.get(key))


    # still cannot check against google.fa.ke/eviladdress
    for value in URLs:
        if not _url_pattern.search(value):
          return False

    return True



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import urlparse
import itertools


def parse_qs(qs):
    """Parse query string, like in urlparse, but do not generate list
    unless the element appears multiple times (same behavior than
    Zope).
    """
    remove_list = lambda e: e[0] if len(e) == 1 else e

    return dict(
        itertools.imap(lambda (k, v): (k, remove_list(v)),
                       urlparse.parse_qs(qs, True).iteritems()))



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from zope.interface.verify import verifyObject
from zope.component import queryUtility

from Products.Silva.testing import tests

from ..interfaces import ICodeSourceService, ICodeSourceInstaller
from ..testing import FunctionalLayer


class ServiceTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addFolder('folder', 'Folder')

    def test_service(self):
        """Verify that the service implements the correct interfaces.
        """
        service = queryUtility(ICodeSourceService)
        self.assertTrue(verifyObject(ICodeSourceService, service))
        self.assertTrue('service_codesources' in self.root.objectIds())
        self.assertEqual(service, self.root.service_codesources)

    def test_default_installed_sources(self):
        """Verify installed sources. By default it should cs_toc and
        cs_citation.
        """
        self.assertTrue('cs_citation' in self.root.objectIds())
        self.assertTrue('cs_toc' in self.root.objectIds())

        service = queryUtility(ICodeSourceService)
        tests.assertContentItemsEqual(
            list(service.get_installed_sources()),
            [self.root.cs_toc, self.root.cs_citation])

        # We can get the installable and they will tell us the same
        candidates = list(service.get_installable_source(
                identifier='cs_citation'))
        self.assertEqual(len(candidates), 1)
        installable = candidates[0]
        self.assertTrue(installable.is_installed(self.root), True)
        self.assertEqual(
            installable.location,
            'Products.SilvaExternalSources:/Products/SilvaExternalSources/codesources/cs_citation')

        candidates = list(service.get_installable_source(
                identifier='cs_toc'))
        self.assertEqual(len(candidates), 1)
        installable = candidates[0]
        self.assertTrue(installable.is_installed(self.root), True)
        self.assertEqual(
            installable.location,
            'Products.SilvaExternalSources:/Products/SilvaExternalSources/codesources/cs_toc')

    def test_clear_and_find_installed_sources(self):
        """Clear and find again installed code sources.
        """
        service = queryUtility(ICodeSourceService)
        installed = list(service.get_installed_sources())
        self.assertEqual(len(installed), 2)

        service.clear_installed_sources()
        installed = list(service.get_installed_sources())
        self.assertEqual(len(installed), 0)

        service.find_installed_sources()
        installed = list(service.get_installed_sources())
        self.assertEqual(len(installed), 2)

    def test_install_source(self):
        """Install cs_portlet_element, a default code source into a
        folder.
        """
        service = queryUtility(ICodeSourceService)
        candidates = list(service.get_installable_source(
                identifier='cs_portlet_element'))
        self.assertEqual(len(candidates), 1)
        installable = candidates[0]
        self.assertTrue(verifyObject(ICodeSourceInstaller, installable))
        self.assertEqual(
            installable.identifier,
            'cs_portlet_element')
        self.assertEqual(
            installable.location,
            'Products.SilvaExternalSources:/Products/SilvaExternalSources/codesources/cs_portlet_element')
        self.assertEqual(installable.validate(), True)

        # By default this source is not installed
        self.assertEqual(installable.is_installed(self.root), False)
        self.assertEqual(installable.is_installed(self.root.folder), False)

        # We can install it.
        installable.install(self.root.folder)

        # And it is installed
        self.assertEqual(installable.is_installed(self.root), False)
        self.assertEqual(installable.is_installed(self.root.folder), True)
        self.assertTrue('cs_portlet_element' in self.root.folder.objectIds())

        # And the location matches the one of the installer
        installed = self.root.folder.cs_portlet_element
        self.assertEqual(installable.location, installed.get_fs_location())
        self.assertItemsEqual(
            installed.objectIds(),
            ['icon.png', 'README', 'LICENSE', 'portlet_element'])
        self.assertEqual(
            installed._getOb('icon.png').meta_type,
            'Image')
        self.assertEqual(
            installed._getOb('portlet_element').meta_type,
            'Page Template')

        # The source appears in the service as well
        tests.assertContentItemsEqual(
            list(service.get_installed_sources()),
            [self.root.cs_citation, self.root.cs_toc, installed])


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ServiceTestCase))
    return suite




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from zope.interface.verify import verifyObject

from Products.Silva.testing import TestRequest
from Products.ZSQLMethods.tests.dummy import DummySQLConnection
from Shared.DC.ZRDB.DA import DatabaseError

from ..interfaces import IExternalSource
from ..testing import FunctionalLayer


class SQLSourceTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')
        factory = self.root.manage_addProduct['silva.app.document']
        factory.manage_addDocument('example', 'Example')
        factory = self.root.manage_addProduct['ZSQLiteDA']
        factory.manage_addZSQLiteConnection('sqlite', 'SQLite', ':memory:')
        self.root._setObject('dummy', DummySQLConnection('dummy', 'Dummy'))

    def test_source(self):
        """Simply add and verify a SQL Source.
        """
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addSQLSource('sql_data', 'SQL Data')
        source = self.root._getOb('sql_data', None)
        self.assertNotEqual(source, None)
        self.assertTrue(verifyObject(IExternalSource, source))
        self.assertItemsEqual(source.objectIds(), ['layout'])
        self.assertEqual(source.is_previewable(), True)
        self.assertEqual(source.is_usable(), True)
        self.assertItemsEqual(
            source.available_connection_ids(),
            [('Dummy (dummy)', 'dummy'),
             ('SQLite (sqlite), which is <font color=red> not connected</font>', 'sqlite')])

    def test_render(self):
        """Test rendering of the sql source.
        """
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addSQLSource('sql_data', 'SQL Data')

        source = self.root._getOb('sql_data', None)
        source._set_connection_id('dummy')
        version = self.root.example.get_editable()

        # The dummy connector is not connected
        with self.assertRaises(DatabaseError):
            source.to_html(version, TestRequest())

        # XXX write test with a DB.

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SQLSourceTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import logging
import lxml.sax

from five import grok

from silva.core import conf as silvaconf
from silva.core.interfaces import IVersion, ISilvaXMLHandler
from silva.core.editor.transform.interfaces import ISilvaXMLImportFilter
from silva.core.editor.transform.base import TransformationFilter
from silva.core.xml import handlers
from zeam.component import getWrapper
from zeam.form.silva.interfaces import IXMLFormSerialization

from silva.core.xml import NS_SILVA_URI

from . import NS_SOURCE_URI
from ..interfaces import IExternalSourceManager, ISourceAsset
from ..errors import SourceError

logger = logging.getLogger('silva.core.xml')

silvaconf.namespace(NS_SOURCE_URI)


class ExternalSourceImportFilter(TransformationFilter):
    grok.adapts(IVersion, ISilvaXMLHandler)
    grok.provides(ISilvaXMLImportFilter)
    grok.order(15)

    def __init__(self, context, handler):
        self.context = context
        self.handler = handler

    def prepare(self, name, text):
        self.sources = getWrapper(self.context, IExternalSourceManager)

    def __call__(self, tree):
        importer = self.handler.getExtra()
        request = importer.request
        for node in tree.xpath(
                '//html:div[contains(@class, "external-source")]',
                namespaces={'html': 'http://www.w3.org/1999/xhtml'}):
            name = node.attrib.get('source-identifier')
            if name is None:
                importer.reportProblem(
                    u"Broken source in import.", self.context)
                continue
            try:
                source = self.sources(request, name=name)
            except SourceError as error:
                importer.reportProblem(
                    u"Broken source in import: {0}".format(error),
                    self.context)
                continue
            identifier = source.new()

            deserializers = getWrapper(
                source, IXMLFormSerialization).getDeserializers()
            for field_node in node.xpath(
                './cs:fields/cs:field', namespaces={'cs': NS_SOURCE_URI}):
                field_id = field_node.attrib['id']
                if field_id.startswith('field-'):
                    # XXX Backward compatiblity 3.0b1
                    field_id = field_id[6:].replace('-', '_')
                deserializer = deserializers.get(field_id)
                if deserializer is None:
                    # This field have been removed. Ignore it.
                    logger.warn(
                        u"Unknown source parameter %s in %s" % (field_id, name))
                    continue
                # Deserialize the value
                deserializer(field_node, self.handler)

            del node[:]
            del node.attrib['source-identifier']
            node.attrib['data-source-instance'] = identifier


class SourceParameterHandler(handlers.SilvaHandler):
    """ Handle source parameter.

    Only to be used by a SourceParametersHandler
    """

    proxy = None

    def startElementNS(self, name, qname, attrs):
        if name == (NS_SOURCE_URI, 'field'):
            self.proxy = lxml.sax.ElementTreeContentHandler()
            self.proxy.startElementNS(name, qname, attrs)
            self.setData('field_id', attrs[(None, 'id')])
        elif self.proxy is not None:
            self.proxy.startElementNS(name, qname, attrs)

    def characters(self, input_text):
        text = input_text.strip()
        if self.proxy is not None and text:
            self.proxy.characters(text)

    def endElementNS(self, name, qname):
        if name == (NS_SOURCE_URI, 'field'):
            self.proxy.endElementNS(name, qname)
            self.parentHandler().deserializeSourceField(
                self.getData('field_id'),
                self.proxy.etree.getroot())
            self.proxy = None
        elif self.proxy is not None:
            self.proxy.endElementNS(name, qname)


class SourceParametersHandler(handlers.SilvaHandler):
    """Handler for importing source parameters.

    The parent handler must define a `source` property
    (IExternalSourceManager) to be used by this handler.

    see SourceAssetVersionHandler for example usage.
    """
    deserializers = None

    def getOverrides(self):
        return {(NS_SOURCE_URI, 'field'): SourceParameterHandler}

    def deserializeSourceField(self, field_id, field_node):
        if self.deserializers is None:
            return
        deserializer = self.deserializers.get(field_id)
        if deserializer is None:
            logger.warn(u"Unknown source parameter %s." % (field_id))
            return
        deserializer(field_node, self)

    def startElementNS(self, name, qname, attrs):
        if name == (NS_SOURCE_URI, 'fields'):
            source = self.parentHandler().source
            if source is not None:
                self.deserializers = getWrapper(
                    source, IXMLFormSerialization).getDeserializers()

    def endElementNS(self, name, qname):
        if name == (NS_SOURCE_URI, 'fields'):
            self.deserializers = None


class SourceHandler(object):
    """Mixin handler to unserialize sources.
    """

    source = None

    def getOverrides(self):
        return {(NS_SOURCE_URI, 'fields'): SourceParametersHandler}

    def createSource(self, identifier, context):
        importer = self.getExtra()
        factory = getWrapper(context, IExternalSourceManager)
        try:
            self.source = factory(importer.request, name=identifier)
        except SourceError, error:
            importer.reportProblem(
                u"Broken source in import: {0}".format(error),
                context)
            return None
        else:
            return self.source.new()

    def getSource(self):
        return self.source


class SourceAssetVersionHandler(SourceHandler, handlers.SilvaVersionHandler):
    source = None

    def _createVersion(self, identifier):
        factory = self.parent().manage_addProduct['SilvaExternalSources']
        factory.manage_addSourceAssetVersion(identifier, '')

    def startElementNS(self, name, qname, attrs):
        if (NS_SILVA_URI, 'content') == name:
            version = self.createVersion(attrs)
            source_identifier = attrs[(None, 'source_identifier')]
            identifier = self.createSource(source_identifier, version)
            if identifier:
                version.set_parameters_identifier(identifier)

    def endElementNS(self, name, qname):
        if (NS_SILVA_URI, 'content') == name:
            self.updateVersionCount()
            self.storeMetadata()
            self.storeWorkflow()
            self.source = None


class SourceAssetHandler(handlers.SilvaHandler):
    silvaconf.name('source_asset')

    def getOverrides(self):
        return {(NS_SILVA_URI, 'content'): SourceAssetVersionHandler}

    def _createContent(self, identifier):
        factory = self.parent().manage_addProduct['SilvaExternalSources']
        factory.manage_addSourceAsset(identifier, '', no_default_version=True)

    def _verifyContent(self, content):
        return ISourceAsset.providedBy(content)

    def startElementNS(self, name, qname, attrs):
        if name == (NS_SOURCE_URI, 'source_asset'):
            self.createContent(attrs)

    def endElementNS(self, name, qname):
        if name == (NS_SOURCE_URI, 'source_asset'):
            self.notifyImport()



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from lxml import etree
from lxml.sax import SaxError, _getNsTag
from xml.sax.handler import ContentHandler


class ElementTreeContentHandler(ContentHandler):
    """Build an lxml ElementTree from SAX-like events. This is a
    simplified version of lxml.sax.ElementTreeContentHandler that
    behave in a compatible fashion with Sprout version.
    """
    def __init__(self, makeelement=None, root=None):
        self._root = root
        self._element_stack = []
        if self._root is not None:
            self._element_stack.append(self._root)
        self._default_ns = None
        self._ns_mapping = {None: [None]}
        self._new_mappings = {}
        if makeelement is None:
            makeelement = etree.Element
        self._makeelement = makeelement

    @property
    def etree(self):
        "Contains the generated ElementTree after parsing is finished."
        return etree.ElementTree(self._root)

    def setDocumentLocator(self, locator):
        pass

    def startDocument(self):
        pass

    def endDocument(self):
        pass

    def startPrefixMapping(self, prefix, uri):
        self._new_mappings[prefix] = uri
        try:
            self._ns_mapping[prefix].append(uri)
        except KeyError:
            self._ns_mapping[prefix] = [uri]
        if prefix is None:
            self._default_ns = uri

    def endPrefixMapping(self, prefix):
        ns_uri_list = self._ns_mapping[prefix]
        ns_uri_list.pop()
        if prefix is None:
            self._default_ns = ns_uri_list[-1]

    def startElementNS(self, ns_name, qname, attributes=None):
        ns_uri, local_name = ns_name
        if ns_uri:
            el_name = "{%s}%s" % ns_name
        elif self._default_ns:
            el_name = "{%s}%s" % (self._default_ns, local_name)
        else:
            el_name = local_name

        if attributes:
            attrs = {}
            for name_tuple, value in attributes.iteritems():
                if isinstance(name_tuple, basestring):
                    attr_name = name_tuple
                elif name_tuple[0]:
                    attr_name = "{%s}%s" % name_tuple
                else:
                    attr_name = name_tuple[1]
                attrs[attr_name] = value
        else:
            attrs = None

        element_stack = self._element_stack
        if self._root is None:
            element = self._root = self._makeelement(
                el_name, attrs, self._new_mappings)
        else:
            element = etree.SubElement(
                element_stack[-1], el_name, attrs, self._new_mappings)
        element_stack.append(element)
        self._new_mappings.clear()

    def processingInstruction(self, target, data):
        pass

    def endElementNS(self, ns_name, qname):
        element = self._element_stack.pop()
        ns_uri, name = ns_name
        if ns_uri is None:
            ns_uri = self._default_ns
        if (ns_uri, name) != _getNsTag(element.tag):
            raise SaxError("Unexpected element closed: {%s}%s" % ns_name)

    def startElement(self, name, attributes=None):
        self.startElementNS((None, name), name, attributes)

    def endElement(self, name):
        self.endElementNS((None, name), name)

    def characters(self, data):
        last_element = self._element_stack[-1]
        try:
            # if there already is a child element, we must append to its tail
            last_element = last_element[-1]
            last_element.tail = (last_element.tail or '') + data
        except IndexError:
            # otherwise: append to the text
            last_element.text = (last_element.text or '') + data

    ignorableWhitespace = characters



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

# Helpers for cs_toc...

from AccessControl import ModuleSecurityInfo
import zope.deferredimport

module_security = ModuleSecurityInfo(
    'Products.SilvaExternalSources.codesources.toc')
module_security.declarePublic('get_publishable_content_types')
module_security.declarePublic('get_container_content_types')

zope.deferredimport.deprecated(
    'Please refresh your TOC code source',
    get_publishable_content_types='Products.SilvaExternalSources.codesources.api:get_publishable_content_types',
    get_container_content_types='Products.SilvaExternalSources.codesources.api:get_container_content_types')





# -*- coding: utf-8 -*-
# Copyright (c) 2013  Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from zeam.component import getWrapper
from zope.interface.verify import verifyObject

from Products.Silva.ftesting import public_settings
from Products.Silva.testing import TestRequest
from silva.core.interfaces import IPublicationWorkflow

from ..testing import FunctionalLayer
from ..interfaces import ISourceAsset, ISourceAssetVersion
from ..interfaces import IExternalSourceManager


class SourceAssetTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')

    def test_empty_source_asset(self):
        """Test an empty (aka broken) Source Asset.
        """
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addSourceAsset('asset', 'Test Asset')
        asset = self.root._getOb('asset', None)
        self.assertTrue(verifyObject(ISourceAsset, asset))
        version = asset.get_editable()
        self.assertTrue(verifyObject(ISourceAssetVersion, version))

        # A source asset can be used as a source, but does nothing
        self.assertEqual(asset.get_parameters_form(), None)
        self.assertEqual(asset.get_icon(), None)
        self.assertEqual(asset.get_description(), 'Broken or missing source.')
        self.assertEqual(asset.is_usable(), False)
        self.assertEqual(asset.is_previewable(), False)
        self.assertEqual(asset.is_cacheable(), False)

        # While rendering of the broken source asset, preview gives a
        # usefull message.
        with self.layer.get_browser(public_settings) as browser:
            browser.login('editor')
            self.assertEqual(browser.open('/root/++preview++/asset'), 200)
            self.assertEqual(browser.inspect.title, [])
            self.assertEqual(
                browser.inspect.content,
                ['External Source unknow is not available.'])

        # We published the asset and it stays not usuable.
        IPublicationWorkflow(asset).publish()
        self.assertEqual(asset.get_parameters_form(), None)
        self.assertEqual(asset.get_icon(), None)
        self.assertEqual(asset.get_description(), 'Broken or missing source.')
        self.assertEqual(asset.is_usable(), False)
        self.assertEqual(asset.is_previewable(), False)
        self.assertEqual(asset.is_cacheable(), False)
        with self.layer.get_browser(public_settings) as browser:
            self.assertEqual(browser.open('/root/asset'), 200)
            self.assertEqual(browser.inspect.title, [])
            self.assertEqual(
                browser.inspect.content,
                ['Sorry, this Silva Source Asset is not viewable.'])

    def test_working_source_asset(self):
        """Test a working source asset.
        """
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addSourceAsset('asset', 'Test Asset')
        asset = self.root._getOb('asset', None)
        version = asset.get_editable()

        # Create a source with cs_citation
        sources = getWrapper(version, IExternalSourceManager)
        controller = sources(TestRequest(), name='cs_citation')
        controller.create()
        controller.getContent().citation = 'Silva is a great CMS.'
        version.set_parameters_identifier(controller.getId())

        # The content is not yet published.
        self.assertEqual(asset.get_parameters_form(), None)
        self.assertEqual(asset.get_icon(), None)
        self.assertEqual(asset.get_description(), 'Broken or missing source.')
        self.assertEqual(asset.is_usable(), False)
        self.assertEqual(asset.is_previewable(), False)
        self.assertEqual(asset.is_cacheable(), False)
        # You can preview it.
        with self.layer.get_browser(public_settings) as browser:
            browser.login('editor')
            self.assertEqual(browser.open('/root/++preview++/asset'), 200)
            self.assertEqual(browser.inspect.title, [])
            self.assertEqual(browser.inspect.content, ['Silva is a great CMS.'])

        # We publish it
        original = self.root.cs_citation
        IPublicationWorkflow(asset).publish()
        self.assertEqual(asset.get_parameters_form(), None)
        self.assertEqual(asset.get_icon(), original.get_icon())
        self.assertEqual(asset.get_description(), original.get_description())
        self.assertEqual(asset.is_usable(), True)
        self.assertEqual(asset.is_previewable(), True)
        self.assertEqual(asset.is_cacheable(), True)
        # Since it is published you can now see it.
        with self.layer.get_browser(public_settings) as browser:
            self.assertEqual(browser.open('/root/asset'), 200)
            self.assertEqual(browser.inspect.title, [])
            self.assertEqual(browser.inspect.content, ['Silva is a great CMS.'])

    def test_broken_render_source_asset(self):
        """Test a source that renders a source with a broken render
        script.
        """
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addSourceAsset('asset', 'Test Asset')
        asset = self.root._getOb('asset', None)
        version = asset.get_editable()

        # Create a test code source
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addCodeSource('brokensource', 'Broken Source', 'script')
        source = self.root._getOb('brokensource', None)

        # Add the rendering script
        factory = source.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script')
        script = source._getOb('script')
        script.write("""
##parameters=model,version,REQUEST
I am broken
return "Render source"
""")

        # Create a source with the new test source
        sources = getWrapper(version, IExternalSourceManager)
        controller = sources(TestRequest(), name='brokensource')
        controller.create()
        version.set_parameters_identifier(controller.getId())

        # You can preview it, but it will fail with a nice message
        with self.layer.get_browser(public_settings) as browser:
            browser.login('editor')
            self.assertEqual(browser.open('/root/++preview++/asset'), 200)
            self.assertEqual(browser.inspect.title, [])
            self.assertEqual(
                browser.inspect.content,
                ['Error while rendering Broken Source\n  '
                 'A programmation error happened while rendering the source.'])

        IPublicationWorkflow(asset).publish()
        # We published the asset we still a nice error message instead
        # of a failure message.
        with self.layer.get_browser(public_settings) as browser:
            browser.options.handle_errors = False
            self.assertEqual(browser.open('/root/asset'), 200)
            self.assertEqual(browser.inspect.title, [])
            self.assertEqual(
                browser.inspect.content,
                ['Sorry, this Silva Source Asset is not viewable.'])


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SourceAssetTestCase))
    return suite



##parameters=model, version, code, archive='', codebase='', width, height, params=''

if archive is None:
    archive_attr = ''
else:
    archive_url = archive.absolute_url()
    archive_attr = 'archive="%s"' % archive_url

if codebase== '':
    codebase_attr = ''
else:
    codebase_url = codebase.absolute_url()
    codebase_attr = 'codebase="%s"' % codebase_url

param_tags = ''
if params != '':
    start = 0
    param_list = []
    while start < len(params):
        i = params.find('=', start)
        if i > -1:
            name = params[start:i].strip()
        else:
            break
        i = params.find('"', i) + 1
        j = params.find('"', i)
        if i > -1 and j > -1:
            value = params[i:j].strip()
        else:
            break
        param_list.append((name, value))
        start = j + 1
    for name, value in param_list:
        param_tags = param_tags + '  <param name="' + name \
                + '" value="' + value + '" />\n'


return """
<p class="p">
<applet code="%s" %s%s width="%s" height="%s" align="baseline">
%s  No Java Applet supported!!
</applet>
</p>
""" % (code, archive_attr, codebase_attr, width, height, param_tags)



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from setuptools import setup, find_packages
import os

version = '3.0.3'

tests_require = [
    'silva.app.document [test]',
    'Products.Silva [test]',
    'Products.ZSQLiteDA',
    'silvatheme.standardissue',
    ]

# Products.ZSQLMethods should be as an option ?

setup(name='Products.SilvaExternalSources',
      version=version,
      description="Externals content sources to embed in Silva CMS",
      long_description=open(os.path.join("Products", "SilvaExternalSources", "README.txt")).read() + "\n" +
                       open(os.path.join("Products", "SilvaExternalSources", "HISTORY.txt")).read(),
      classifiers=[
          "Framework :: Zope2",
          "License :: OSI Approved :: BSD License",
          "Programming Language :: Python",
          "Topic :: Software Development :: Libraries :: Python Modules",
      ],
      keywords='silva cms document external sources',
      author='Infrae',
      author_email='info@infrae.com',
      url='https://github.com/silvacms/Products.SilvaExternalSources',
      license='BSD',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'Products.Formulator',
        'Products.Silva',
        'Products.SilvaMetadata',
        'Products.ZSQLMethods',
        'collective.monkeypatcher',
        'five.grok',
        'grokcore.chameleon',
        'lxml',
        'setuptools',
        'silva.core.conf',
        'silva.core.editor',
        'silva.core.editor',
        'silva.core.interfaces',
        'silva.core.services',
        'silva.core.views',
        'silva.core.xml',
        'silva.translations',
        'silva.ui',
        'zeam.component',
        'zeam.form.silva',
        'zeam.utils.batch',
        'zope.annotation',
        'zope.component',
        'zope.event',
        'zope.i18n',
        'zope.interface',
        'zope.intid',
        'zope.lifecycleevent',
        'zope.publisher',
        'zope.schema',
        ],
      entry_points="""
      [zodbupdate]
      renames = Products.SilvaExternalSources:CLASS_CHANGES
      [Products.SilvaExternalSources.sources]
      defaults = Products.SilvaExternalSources.codesources
      [silva.core.editor.extension]
      source = Products.SilvaExternalSources.editor:extension
      [silva.ui.resources]
      source = Products.SilvaExternalSources.editor:IEditorPluginResources
      """,
      tests_require = tests_require,
      extras_require = {'test': tests_require},
      )



##parameters=REQUEST, model, version, ref, width, height, play='true', quality='high', params=''

asset_url = ref.absolute_url()
if width == '':
    width_attr = ''
else:
    width_attr = 'width="%s" ' % width

if height == '':
    height_attr = ''
else:
    height_attr = 'height="%s" ' % height

param_tags = ''
param_attrs = ''
if params != '':
    start = 0
    param_list = []
    while start < len(params):
        i = params.find('=', start)
        if i > -1:
            name = params[start:i].strip()
        else:
            break
        i = params.find('"', i) + 1
        j = params.find('"', i)
        if i > -1 and j > -1:
            value = params[i:j].strip()
        else:
            break
        param_list.append((name, value))
        start = j + 1
    for name, value in param_list:
        param_tags = param_tags + '\n  <param name="' + name \
                + '" value="' + value + '" />'
        param_attrs = param_attrs + '\n    ' + name + '="' + value + '" '

# the loop parameter is not here anymore, it was not working with some animations.

return """
<p class="p">
<object %s%s
  classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"
  codebase="http://active.macromedia.com/flash5/cabs/swflash.cab#version=5,0,0,0">
  <param name="movie" value="%s" />
  <param name="play" value="%s" />
  <param name="quality" value="%s" />%s
  <comment>
  <embed %s%ssrc="%s" play="%s" quality="%s"
    pluginspage="http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash"
    swLiveConnect="true"%s />
  </comment>
</object>
</p>
""" % (width_attr, height_attr, asset_url, play, quality, param_tags,
       width_attr, height_attr, asset_url, play, quality, param_attrs)

return """
<p class="p">
<object %s%s data="%s" type="application/x-shockwave-flash">
  <param name="movie" value="%s" />
  <param name="play" value="%s" />
  <param name="quality" value="%s" />%s
</object>
</p>
""" % (width_attr, height_attr, asset_url,
       asset_url, play, quality, param_tags)



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import os

from five import grok
from zope.component import getMultiAdapter

# Zope
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from OFS.Folder import Folder

from Products.ZSQLMethods.SQL import SQLConnectionIDs, SQL
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
# Formulator
from Products.Formulator.Form import ZMIForm
from Products.Formulator.XMLToForm import XMLToForm
# Silva
from Products.Silva.SilvaPermissions import ViewManagementScreens, \
    AccessContentsInformation
from Products.Silva.helpers import add_and_edit
from Products.SilvaExternalSources.interfaces import IExternalSource
from Products.SilvaExternalSources.ExternalSource import EditableExternalSource

from silva.core import conf as silvaconf
from silva.core.interfaces import IVersion
from silva.core.services.base import ZMIObject
from zeam.utils.batch import Batch, IBatching

DATA_PATH = os.path.join(os.path.dirname(__file__), 'layout')


class SQLSource(EditableExternalSource, Folder, ZMIObject):
    grok.implements(IExternalSource)
    # register icon and factories
    silvaconf.icon('www/sqlsource.png')
    silvaconf.factory('manage_addSQLSourceForm')
    silvaconf.factory('manage_addSQLSource')
    silvaconf.zmi_addable(True)

    meta_type = "Silva SQL Source"
    security = ClassSecurityInfo()

    _sql_method_id = 'sql_method'
    _layout_id = 'layout'
    _default_batch_size = 10
    _v_cached_parameters = None

    # ZMI Tabs
    manage_options = (
        {'label':'Edit', 'action':'editSQLSource'},
        {'label':'Parameters', 'action':'parameters/manage_main'},
        ) + Folder.manage_options
    management_page_charset = 'utf-8'

    security.declareProtected(ViewManagementScreens, 'editSQLSource')
    editSQLSource = PageTemplateFile(
        'www/sqlSourceEdit', globals(),  __name__='sqlCodeSource')

    def __init__(self, id):
        self.id = id
        self._sql_method = None
        self._statement = None
        self._connection_id = None

    # ACCESSORS

    def layout_id(self):
        return self._layout_id

    def connection_id(self):
        return self._connection_id

    def statement(self):
        return self._statement

    def available_connection_ids(self):
        return SQLConnectionIDs(self)

    security.declareProtected(AccessContentsInformation, 'to_html')
    def to_html(self, content, request, **parameters):
        """ render HTML for SQL source
        """
        values = self._get_data(parameters)
        names = values.names()
        batch_size = self._default_batch_size
        if parameters.get('sqlbatchsize'):
            batch_size = int(parameters.get('sqlbatchsize'))
        data = Batch(
            values.dictionaries(),
            factory=self._decode_dict_helper,
            count=batch_size,
            name=self.getId(),
            request=request)
        model = content
        if IVersion.providedBy(content):
            model = content.get_content()
        layout = self._getOb(self._layout_id)
        batch = getMultiAdapter((model, data, request), IBatching)()
        return layout(
            table=data, batch=batch, names=names, parameters=parameters)

    def _get_data(self, args):
        if not self._sql_method:
            self._set_sql_method()
        elif self._v_cached_parameters != self.get_parameters_form().get_field_ids():
            self._set_sql_method()
        args = self._encode_dict_helper(args)
        return self._sql_method(REQUEST=args)

    def _decode_dict_helper(self, dictionary):
        for key, value in dictionary.items():
            if type(value) is type(''):
                dictionary[key] = unicode(
                    value, self._data_encoding, 'replace')
        return dictionary

    def _encode_dict_helper(self, dictionary):
        for key, value in dictionary.items():
            if type(value) is type(u''):
                dictionary[key] =  value.encode(
                    self._data_encoding, 'replace')
        return dictionary

    # MODIFIERS

    def _set_statement(self, statement):
        self._statement = statement
        #invalidate sql method
        self._sql_method = None
        self._p_changed = 1

    def _set_connection_id(self, id):
        self._connection_id = id
        #invalidate sql method
        self._sql_method = None
        self._p_changed = 1

    def _set_sql_method(self):
        self._v_cached_parameters = parameters = self.get_parameters_form().get_field_ids()
        arguments = '\n'.join(parameters)
        self._sql_method = SQL(
            self._sql_method_id, '', self._connection_id,
            arguments.encode('ascii'), self._statement.encode('UTF-8'))
        self._p_changed = 1

    # MANAGERS

    security.declareProtected(ViewManagementScreens, 'manage_editSQLSource')
    def manage_editSQLSource(
        self, title, data_encoding, statement, connection_id=None,
        description=None, cacheable=None, layout_id=None, reset_layout=None,
        reset_params=None, previewable=None, usable=None,
        ):
        """ Edit SQLSource object
        """
        msg = u''

        if reset_layout:
            reset_table_layout(self)
            return self.editSQLSource(
                manage_tabs_message="Table rendering pagetemplate reset " \
                    "to default layout.")

        if reset_params:
            reset_parameter_form(self)
            return self.editSQLSource(
                manage_tabs_message="Parameters form reset to default.")

        if data_encoding and data_encoding != self._data_encoding:
            try:
                unicode('abcd', data_encoding, 'replace')
            except LookupError:
                # unknown encoding, return error message
                msg  += "Unknown encoding %s, not changed!" % data_encoding
                return self.editSQLSource(manage_tabs_message=msg)
            self.set_data_encoding(data_encoding)
            msg += "Data encoding changed. "

        if connection_id and connection_id != self._connection_id:
            self._set_connection_id(connection_id)
            msg += "Connection id changed. "

        if statement:
            statement = unicode(statement, 'UTF-8')
            self._set_statement(statement)
            msg += "SQL statement changed. "

        title = unicode(title, self.management_page_charset)
        if title and title != self.title:
            self.set_title(title)
            msg += "Title changed. "

        description = unicode(description, self.management_page_charset)
        if description != self._description:
            self.set_description(description)
            msg += "Description changed. "

        if not bool(cacheable) is self.is_cacheable():
            self.set_cacheable(bool(cacheable))
            msg += "Cacheability setting changed. "

        if not bool(previewable) is self.is_previewable():
            self.set_previewable(bool(previewable))
            msg += "Previewable setting changed. "

        if not bool(usable) is self.is_usable():
            self.set_usable(bool(usable))
            msg += "Usability setting changed. "

        if layout_id and layout_id != self._layout_id:
            self._layout_id = layout_id
            msg += "Layout object id changed. "

        return self.editSQLSource(manage_tabs_message=msg)

InitializeClass(SQLSource)


manage_addSQLSourceForm = PageTemplateFile(
    "www/sqlSourceAdd", globals(), __name__='manage_addSQLSourceForm')

def manage_addSQLSource(context, id, title=None, REQUEST=None):
    """Add a SQLSource object
    """
    source = SQLSource(id)
    title = unicode(title, source.management_page_charset)
    source.title = title
    context._setObject(id, source)
    source = context._getOb(id)
    source._set_statement('SELECT <dtml-var columns> FROM <dtml-var table>')
    # parameters form
    reset_parameter_form(source)
    reset_table_layout(source)
    add_and_edit(context, id, REQUEST, screen='editSQLSource')
    return ''

def reset_table_layout(source):
    # Works for Zope object implementing a 'write()" method...
    layout = [
        ('layout', ZopePageTemplate, 'sqltable.zpt'),
    ]

    for identifier, factory, filename in layout:
        with open(os.path.join(DATA_PATH, filename), 'rb') as data:
            if identifier not in source.objectIds():
                source._setObject(identifier, factory(identifier))
            source._getOb(identifier).write(data.read())

def reset_parameter_form(source):
    with open(os.path.join(DATA_PATH, 'sqlparameters.xml')) as data:
        form = ZMIForm('form', 'Parameters form', unicode_mode=1)
        XMLToForm(data.read(), form)
        source.set_form(form)



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest
import doctest

from zope.interface.verify import verifyObject
from zeam.component import getWrapper
from silva.core.views import views as silvaviews
from silva.core.interfaces import IFolder

from Products.Silva.testing import TestRequest
from silvatheme.standardissue.standardissue import IStandardIssueSkin

from ..interfaces import availableSources
from ..interfaces import ICodeSource, IExternalSource
from ..interfaces import IExternalSourceController
from ..interfaces import IExternalSourceManager
from ..errors import SourcePreviewError, SourceMissingError
from ..testing import FunctionalLayer


class DefaultCodeSourceTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')

    def test_default_code_source_citation(self):
        """Test default code source implementation.
        """
        source = self.root._getOb('cs_citation')
        self.assertTrue(verifyObject(ICodeSource, source))
        self.assertTrue(verifyObject(IExternalSource, source))
        self.assertEqual(source.is_previewable(), True)
        self.assertEqual(source.is_usable(), True)
        self.assertNotEqual(source.get_fs_location(), None)
        self.assertNotEqual(source.get_parameters_form(), None)
        # The source is available
        self.assertItemsEqual(
            filter(lambda (name, candidate): name == source.id,
                   availableSources(self.root)),
            [(source.id, source)])

        # By default this source should work
        self.assertEqual(source.test_source(), None)

    def test_default_code_source_toc(self):
        """Test default code source implementation.
        """
        source = self.root._getOb('cs_toc')
        self.assertTrue(verifyObject(ICodeSource, source))
        self.assertTrue(verifyObject(IExternalSource, source))
        self.assertEqual(source.is_previewable(), True)
        self.assertEqual(source.is_usable(), True)
        self.assertNotEqual(source.get_fs_location(), None)
        self.assertNotEqual(source.get_parameters_form(), None)
        # The source is available
        self.assertItemsEqual(
            filter(lambda (name, candidate): name == source.id,
                   availableSources(self.root)),
            [(source.id, source)])

        # By default this source should work
        self.assertEqual(source.test_source(), None)

    def test_add_code_source(self):
        """Add a source, and verify its initial parameters.
        """
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addCodeSource('codesource', 'A Code Source', 'script')

        source = self.root._getOb('codesource', None)
        self.assertTrue(verifyObject(ICodeSource, source))
        self.assertTrue(verifyObject(IExternalSource, source))
        # The source script be broken because of the missing script script.
        self.assertIsNot(source.test_source(), None)
        self.assertEqual(len(source.test_source()), 1)

        # Add the rendering script
        factory = source.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script')
        script = source._getOb('script')
        script.write("""
##parameters=model,version,REQUEST

return "Render source"
""")

        # Now verify the source.
        self.assertEqual(source.test_source(), None)
        self.assertEqual(source.is_usable(), True)
        self.assertEqual(source.is_previewable(), True)
        self.assertEqual(source.get_script_id(), 'script')
        self.assertEqual(source.get_script_layers(), '')
        self.assertEqual(source.get_icon(), None)
        self.assertEqual(source.get_description(), '')
        self.assertEqual(source.test_source(), None)
        # An empty parameter form is created
        parameters = source.get_parameters_form()
        self.assertNotEqual(parameters, None)
        self.assertEqual(len(parameters.get_fields()), 0)
        # This code source was not created from the filesystem
        self.assertEqual(source.get_fs_location(), None)
        # The source is available
        self.assertItemsEqual(
            filter(lambda (name, candidate): name == source.id,
                   availableSources(self.root)),
            [(source.id, source)])


class CodeSourceTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('manager')

        # Create a test document.
        factory = self.root.manage_addProduct['silva.app.document']
        factory.manage_addDocument('example', 'Example')

        # Create a test code source.
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addCodeSource('codesource', 'Test Source', 'script')
        source = self.root._getOb('codesource', None)

        # Add the rendering script
        factory = source.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('script')
        script = source._getOb('script')
        script.write("""
##parameters=model,version,REQUEST

return "Render source"
""")

        # Create an instance of the code source
        version = self.root.example.get_editable()
        sources = getWrapper(version, IExternalSourceManager)
        controller = sources(TestRequest(), name='codesource')
        controller.create()
        self._instance = controller.getId()

    def get_controller(self, layers=[]):
        version = self.root.example.get_editable()
        sources = getWrapper(version, IExternalSourceManager)
        return sources(TestRequest(layers=layers), instance=self._instance)

    def test_controller(self):
        """Test the controller for our source.
        """
        source = self.root.codesource
        controller = self.get_controller()
        self.assertTrue(verifyObject(IExternalSourceController, controller))

        self.assertEqual(controller.getSourceId(), source.id)
        self.assertEqual(controller.label, source.get_title())
        self.assertEqual(controller.description, source.get_description())
        self.assertEqual(controller.editable(), False)
        self.assertEqual(controller.render(), "Render source")

    def test_layers(self):
        """Test that you can set different script for different layers.
        """
        source = self.root.codesource
        # Source is working and no script layers by default
        self.assertIs(source.test_source(), None)
        self.assertEqual(source.get_script_layers(), '')

        # Invalid format
        with self.assertRaises(ValueError):
            source.set_script_layers("""
test:Standard Issue
this is wrong man""")
        self.assertEqual(source.get_script_layers(), '')

        # Invalid skin
        with self.assertRaises(ValueError):
            source.set_script_layers("""
test:Standard Issue
this:Test issue""")
        self.assertEqual(source.get_script_layers(), '')

        # Correctly set a script layer, but the source is broken, the
        # script is missing.
        source.set_script_layers("test:Standard Issue")
        self.assertEqual(source.get_script_layers(), "test:Standard Issue")
        self.assertIsNot(source.test_source(), None)
        self.assertEqual(len(source.test_source()), 1)

        factory = source.manage_addProduct['PythonScripts']
        factory.manage_addPythonScript('test')
        script = source._getOb('test')
        script.write("""
##parameters=model,version,REQUEST

return "Render layer"
""")
        self.assertIs(source.test_source(), None)

        # Without any layer, you get the default rendering.
        controller = self.get_controller()
        self.assertTrue(verifyObject(IExternalSourceController, controller))
        self.assertEqual(controller.render(), "Render source")

        # With the given skin, you should have the new rendering
        controller = self.get_controller(layers=[IStandardIssueSkin])
        self.assertTrue(verifyObject(IExternalSourceController, controller))
        self.assertEqual(controller.render(), "Render layer")

    def test_archive(self):
        """Test you can move the source in a folder. The rendering in
        the document will be broken after, and you will have a
        message, only in preview.
        """
        factory = self.root.manage_addProduct['OFSP']
        factory.manage_addFolder('folder', 'Folder')
        self.assertFalse(IFolder.providedBy(self.root.folder))
        self.assertNotIn('codesource', self.root.folder.objectIds())
        self.assertIn('codesource', self.root.objectIds())

        token = self.root.manage_cutObjects(['codesource'])
        self.root.folder.manage_pasteObjects(token)
        self.assertIn('codesource', self.root.folder.objectIds())
        self.assertNotIn('codesource', self.root.objectIds())

        controller = self.get_controller()
        self.assertTrue(verifyObject(IExternalSourceController, controller))
        with self.assertRaises(SourceMissingError) as error:
            controller.render()

        message = silvaviews.render(error.exception, TestRequest())
        self.assertEqual(message.strip(), '')
        message = silvaviews.render(
            error.exception, TestRequest(), preview=True)
        self.assertEqual(
            message.strip(),
            '<p>External Source unknow is not available.</p>')

        # The source is no longer available
        self.assertItemsEqual(
            filter(lambda (name, candidate): name == 'codesource',
                   availableSources(self.root)),
            [])

    def test_usable(self):
        """Test that switching to not usable remove the source from
        the available list.
        """
        source = self.root.codesource
        self.assertEqual(source.is_usable(), True)
        # The source is available
        self.assertItemsEqual(
            filter(lambda (name, candidate): name == source.id,
                   availableSources(self.root)),
            [(source.id, source)])

        source.set_usable(False)
        self.assertEqual(source.is_usable(), False)
        # The source is no longer available
        self.assertItemsEqual(
            filter(lambda (name, candidate): name == source.id,
                   availableSources(self.root)),
            [])

    def test_previewable(self):
        """Test that the preview prevent a source to be rendered if
        preview is asked.
        """
        source = self.root.codesource
        controller = self.get_controller()
        self.assertTrue(verifyObject(IExternalSourceController, controller))
        self.assertEqual(source.is_previewable(), True)
        self.assertEqual(controller.render(preview=True), "Render source")

        source.set_previewable(False)
        controller = self.get_controller()
        self.assertTrue(verifyObject(IExternalSourceController, controller))
        self.assertEqual(source.is_previewable(), False)
        with self.assertRaises(SourcePreviewError) as error:
            self.assertEqual(controller.render(preview=True))

        preview = silvaviews.render(error.exception, TestRequest())
        self.assertIsInstance(preview, basestring)
        self.assertIn('Test Source',  preview)
        self.assertNotEqual(preview, "Render source")
        # You can still render the not preview
        self.assertEqual(controller.render(), "Render source")


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(DefaultCodeSourceTestCase))
    suite.addTest(unittest.makeSuite(CodeSourceTestCase))
    suite.addTest(doctest.DocTestSuite(
            'Products.SilvaExternalSources.codesources.youtube'))
    suite.addTest(doctest.DocTestSuite(
            'Products.SilvaExternalSources.codesources.googlemaps'))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from zope.interface.verify import verifyObject
from Products.Silva.testing import TestRequest, tests

from ..interfaces import ICSVSource, IExternalSource
from ..testing import FunctionalLayer


class CSVSourceTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('author')
        factory = self.root.manage_addProduct['SilvaExternalSources']
        factory.manage_addCSVSource('csv_data', 'CSV Data')
        factory = self.root.manage_addProduct['silva.app.document']
        factory.manage_addDocument('example', 'Example')

    def test_source(self):
        """Simply add and verify a CSV Source.
        """
        source = self.root._getOb('csv_data', None)
        self.assertNotEqual(source, None)
        self.assertTrue(verifyObject(ICSVSource, source))
        self.assertTrue(ICSVSource.extends(IExternalSource))
        self.assertItemsEqual(source.objectIds(), ['layout'])
        self.assertEqual(source.is_previewable(), True)
        self.assertEqual(source.is_usable(), True)
        self.assertEqual(source.get_file_size(), 0)
        self.assertEqual(source.get_filename(), 'csv_data.csv')
        self.assertEqual(source.get_mime_type(), 'text/csv')

        with self.layer.open_fixture('informations.csv') as data:
            with tests.assertTriggersEvents('ObjectModifiedEvent'):
                source.set_file(data)

        self.assertEqual(source.get_file_size(), 281)
        self.assertEqual(source.get_filename(), 'csv_data.csv')
        self.assertEqual(source.get_mime_type(), 'text/csv')

    def test_render(self):
        """Test rendering the CSV source.
        """
        source = self.root._getOb('csv_data', None)
        version = self.root.example.get_editable()
        with self.layer.open_fixture('informations.csv') as data:
            source.set_file(data)

        rendered = source.to_html(version, TestRequest())
        self.assertNotEqual(rendered, '')
        self.assertIn('Update CSV tests', rendered)



def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CSVSourceTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import Acquisition

from five import grok
from zope.interface import Interface, Attribute
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from silva.core.interfaces import ISilvaService, ISilvaLocalService
from silva.core.interfaces import IXMLZEXPExportable
from silva.core.interfaces import IVersionedNonPublishable, IVersion
from silva.core.interfaces import IAsset, IRoot


def availableSources(context):
    """List available sources in the site starting at context.
    """
    sources = {}
    while context is not None:
        for item in context.objectValues():
            if (IExternalSource.providedBy(item) and
                item.id not in sources and item.is_usable()):
                sources[item.id] = item
        if IRoot.providedBy(context):
            break
        context = Acquisition.aq_parent(context)
    sources = sources.items()
    sources.sort(key=lambda i: i[1].get_title().lower())
    return sources


@grok.provider(IContextSourceBinder)
def source_source(context):

    def make_term(identifier, source):
        return SimpleTerm(value=source,
                          token=identifier,
                          title=unicode(source.get_title()))

    return SimpleVocabulary([make_term(*t) for t in availableSources(context)])


class IExternalSource(Interface):
    """ Access to an external source of data.
    """

    # ACCESSORS

    def get_parameters_form():
        """ Returns a Formulator form describing the paramters used by
        the external source or None if not applicable.

        This Formulator form is used in the Silva Document 'external data'
        document element to render the parameters UI.
        """

    def to_html(content, request, **parameters):
        """ Render the HTML for inclusion in the rendered Silva HTML.
        """

    def get_title():
        """Returns the title of this instance.
        """

    def get_icon():
        """Returns an icon associated to the code source. This would
        be a ZODB object.
        """

    def get_description():
        """Returns the purpose of this external source.

        The description is shown in the 'external data' element's editor.
        It can contain a description of the use of its parameters and the
        what data is will render in the document.
        """

    def is_previewable(**parameters):
        """Specify the previewability (in kupu) of the source
        """

    def is_cacheable(**parameters):
        """Returns the cacheability (true or false) for this source.

        Silva Document atempts to cache the public rendering. If a document
        references this external source, it will check for its cachability.
        If the data from this source can be cached this source will only be
        called once.
        """

    def is_usable():
        """Returns True if the source is usable in new contents.
        """


class IEditableExternalSource(IExternalSource):
    """An external source where settings can be edited.
    """

    def set_description(description):
        """Set the description of the external source.
        """

    def set_data_encoding(encoding):
        """Set the output encoding of the external source.
        """

    def set_cacheable(cacheable):
        """Set cacheablility of the external source (boolean). If set
        to False, the output of the source should never be cached.
        """

    def set_previewable(previewable):
        """Set previewablility of the external source in the WYSIWYG (boolean).
        """

    def set_usable(usable):
        """Set if the source is usable in new content.
        """


class ICodeSource(IEditableExternalSource, IXMLZEXPExportable):
    """Code source: an external source built in ZMI.
    """

    def get_data_encoding():
        """ Returns the encoding of source's data.

        Silva expects unicode for its document data. This parameter
        specifies the encoding of the original data so it can be properly
        converted to unicode when passing the data to the Silva Document.

        NOTE: This is usually only used *within* the code source
        implementation.
        """

    def test_source():
        """Test if the source is working or if it has problems. It
        should return None if there are no problems.
        """


class ISourceAssetVersion(IVersion):
    """A version of a source asset.
    """

    def get_controller(request):
        """Return the controller associated to the source.
        """

    def get_source():
        """Return the original source object associated to this asset,
        or raise a SourceError exception.
        """


class ISourceAsset(IVersionedNonPublishable, IExternalSource):
    """Source asset store a external source and parameters to render it
    """

    def get_viewable_source():
        """Return the original source object associated with the
        published version of the asset, or None if it is available or
        broken.
        """


class ISourceEditableVersion(IVersion):
    """A version of a content using the editor external source features
    """

# Code source Service support.

class ICodeSourceService(ISilvaService, ISilvaLocalService):
    """Code source service.
    """

    def find_installed_sources():
        """Find all installed code sources. You can after call
        ``get_installed_sources`` to get the list of installed code
        sources.
        """

    def get_installed_sources():
        """Return all installed code sources.
        """

    def clear_installed_sources():
        """Clear the known list of installed code sources.
        """

    def get_installable_sources():
        """Return a list of all known installable code sources in this
        Silva site.
        """

    def get_installable_source(identifier=None, location=None):
        """Return a specific installable code source in this Silva
        site, defined either by its identifier or location, or return
        None if no installable code source matches.
        """


class ICodeSourceInstaller(Interface):
    """Install or update a specific code source.
    """
    identifier = Attribute(u"Source identifier")
    title = Attribute(u"User-friendly source title")
    script_id = Attribute(u"Script identifier used to render the source")
    description = Attribute(u"User-friendly source description")
    location = Attribute(u"Filesystem location, relative to the egg")

    def validate():
        """Return True if the source is correctly defined on the
        filesystem, that no required information is missing.
        """

    def is_installed(folder):
        """Return True if the source is installed in the given Silva
        container.
        """

    def install(folder):
        """Install in a specific Silva container an instance for this
        source, from the filesystem.
        """

    def update(source):
        """Update from the filesystem a specific source.
        """


class ICSVSource(IEditableExternalSource, IAsset):
    """An external source showing the content of a CSV file.
    """


# This define a parameter instance of a source.


class IExternalSourceInstance(Interface):
    """Store parameters for a given source instance.
    """

    def get_source_identifier():
        """Return the source identifier corresponding to this
        source parameters.
        """

    def get_parameter_identifier():
        """Return an identfier that identifies uniquely those
        parameters.
        """

    def get_source_template():
        """Return a string used a template to warp around the result
        of the source.
        """

    def set_source_template(template):
        """Set a string to use as a template to warp around the result
        of the source.
        """


class IExternalSourceController(Interface):
    """This control an instance of a code source.

    This behave and follow the same API than a Silva form.
    """
    label = Attribute(u'Title of the code source instance')
    description = Attribute(u'Description of the code source instance')

    def new():
        """Create a new instance of the code source on the context
        object.
        """

    def copy(destination):
        """This copy the current instance of the code source into a
        another external source controller.
        """

    def create():
        """Create a new instance of the code source on the context
        object, and save the code source parameters from the current
        request into this new instance.
        """

    def save():
        """Save the code source parameters from the current request.
        """

    def remove():
        """This remove the current instance of the code source from
        the context object.
        """

    def render(view=False, preview=False):
        """Render the instance of the code source, with its parameters.

        ``preview`` if true will render the code source in preview
        mode (paying attention if the code source provides parameters
        in the request, and if it is previewable).

        ``view`` is an option for compatiblity with the content layout
        extension.
        """


class IExternalSourceManager(Interface):
    """Manage external source instances on an context object.
    """

    def new(source):
        """Create and return a new external source instance object to
        store a parameters for a new instance of this source on the
        context object.
        """

    def all():
        """Return all the identifiers of each parameter object
        available on the context object.
        """

    def remove(identifier):
        """Remove the parameter object associated with the given
        ``identifier`` from the context object.
        """

    def get_parameters(instance=None, name=None):
        """Return the parameters and the source associated to this
        instance or source.
        """

    def __call__(request, instance=None, name=None):
        """Retrieve the parameters and the source associated to this
        instance or source, and bind them into a controller together
        with a request.
        """


class ISourceErrors(Interface):

    def report(info):
        """Report an error.
        """

    def fetch():
        """Return a list of all reported errors.
        """




## Script (Python) "ustr"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=s
##title=
##
if not same_type(s,u''):
  s = unicode(s,'utf-8')
return s




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import urllib

from five import grok
from zope import schema
from zope.event import notify
from zope.interface import Interface
from zope.lifecycleevent import ObjectModifiedEvent
from zope.i18n import translate

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Acquisition import aq_base

from Products.Silva.Version import Version
from Products.Silva.VersionedContent import VersionedNonPublishable
from Products.Silva import SilvaPermissions as permissions

from silva.core import conf as silvaconf
from silva.core.conf.interfaces import ITitledContent
from silva.core.views import views as silvaviews
from silva.translations import translate as _
from silva.ui.rest import RedirectToPage
from zeam.component import getWrapper
from zeam.form import silva as silvaforms

from .interfaces import IExternalSourceManager
from .interfaces import ISourceAsset, ISourceAssetVersion, source_source
from .errors import SourceError


class SourceAssetVersion(Version):
    grok.implements(ISourceAssetVersion)
    security = ClassSecurityInfo()
    meta_type = "Silva Source Asset Version"

    _parameter_identifier = None

    security.declarePrivate('set_parameter_identifier')

    def set_parameters_identifier(self, identifier):
        self._parameter_identifier = identifier
        if hasattr(aq_base(self), '_v_original_source'):
            delattr(self, '_v_original_source')

    security.declarePrivate('get_controller')

    def get_controller(self, request):
        manager = getWrapper(self, IExternalSourceManager)
        return manager(request, instance=self._parameter_identifier)

    security.declarePrivate('get_source')

    def get_source(self):
        if not hasattr(aq_base(self), '_v_original_source'):
            manager = getWrapper(self, IExternalSourceManager)
            _, source = manager.get_parameters(self._parameter_identifier)
            self._v_original_source = source
        return self._v_original_source

    security.declarePrivate('get_parameters')

    def get_parameters(self):
        manager = getWrapper(self, IExternalSourceManager)
        parameters, _ = manager.get_parameters(self._parameter_identifier)
        return parameters

InitializeClass(SourceAssetVersion)


class SourceAsset(VersionedNonPublishable):
    """A source asset stores a external source and a set of its parameters.x
    """
    meta_type = "Silva Source Asset"
    grok.implements(ISourceAsset)
    silvaconf.icon('www/code_source_asset.png')
    silvaconf.version_class(SourceAssetVersion)
    security = ClassSecurityInfo()

    security.declareProtected(
        permissions.AccessContentsInformation, 'get_parameters_form')

    def get_parameters_form(self):
        return None

    security.declarePrivate('get_source')

    def get_viewable_source(self):
        viewable = self.get_viewable()
        if viewable is not None:
            try:
                return viewable.get_source()
            except SourceError:
                pass
        return None

    security.declareProtected(
        permissions.AccessContentsInformation, 'to_html')

    def to_html(self, content, request, **parameters):
        viewable = self.get_viewable()
        if viewable is not None:
            return viewable.get_controller(request).render()
        # Should we put an error message instead ?
        return u''

    def get_description(self):
        source = self.get_viewable_source()
        if source is not None:
            return source.get_description()
        return _('Broken or missing source.')

    security.declareProtected(
        permissions.AccessContentsInformation, 'get_icon')

    def get_icon(self):
        source = self.get_viewable_source()
        if source is not None:
            return source.get_icon()
        return None

    security.declareProtected(
        permissions.AccessContentsInformation, 'is_usable')

    def is_usable(self):
        source = self.get_viewable_source()
        if source is not None:
            return True
        return False

    security.declareProtected(
        permissions.AccessContentsInformation, 'is_previewable')

    def is_previewable(self, **parameters):
        source = self.get_viewable_source()
        if source is not None:
            return source.is_previewable()
        return False

    security.declareProtected(
        permissions.AccessContentsInformation, 'is_cacheable')

    def is_cacheable(self, **parameters):
        source = self.get_viewable_source()
        if source is not None:
            return source.is_cacheable()
        return False


InitializeClass(SourceAsset)


class ISourceSelectField(Interface):
    source = schema.Choice(
        title=_(u"Select an external source"),
        source=source_source,
        required=True)


class SourceAssetAddForm(silvaforms.SMIAddForm):
    """Add form for a link.
    """
    grok.context(ISourceAsset)
    grok.name(u'Silva Source Asset')

    fields = silvaforms.Fields(ISourceSelectField)
    titleFields = silvaforms.Fields(ITitledContent)
    actions = silvaforms.Actions(silvaforms.CancelAddAction())
    source = None

    def publishTraverse(self, request, name):
        factory = getWrapper(self.context, IExternalSourceManager)
        try:
            self.source = factory(request, name=urllib.unquote(name))
        except SourceError:
            parent = super(SourceAssetAddForm, self)
            return parent.publishTraverse(request, name)
        self.fields = self.titleFields
        self.__name__ = '/'.join((self.__name__, name))
        return self

    @property
    def formErrors(self):
        if self.source is not None:
            return self.source.formErrors
        return []

    def updateWidgets(self):
        super(SourceAssetAddForm, self).updateWidgets()
        if self.source is not None:
            self.fieldWidgets.extend(
                self.source.fieldWidgets(
                    ignoreRequest=False, ignoreContent=True))

    @silvaforms.action(
        _(u"Select source"),
        available=lambda form: form.source is None,
        implements=silvaforms.IDefaultAction,
        accesskey=u'ctrl+s')
    def select_source(self):
        data, errors = self.extractData()
        if errors:
            return silvaforms.FAILURE
        raise RedirectToPage(sub_tab=data['source'].getId())

    @silvaforms.action(
        _(u"Save"),
        available=lambda form: form.source is not None,
        implements=silvaforms.IDefaultAction,
        accesskey=u'ctrl+s')
    def save(self):
        data, errors = self.extractData()
        source_data, source_errors = self.source.extractData()
        if errors or source_errors:
            return silvaforms.FAILURE
        try:
            content = self._add(self.context, data)
        except ValueError, error:
            self.send_message(error.args[0], type=u"error")
            return silvaforms.FAILURE
        editable = content.get_editable()
        factory = getWrapper(editable, IExternalSourceManager)
        source = factory(self.request, name=self.source.getSourceId())
        source.create()
        editable.set_parameters_identifier(source.getId())
        notify(ObjectModifiedEvent(content))
        self.send_message(_(u"Source Asset added."), type="feedback")
        raise RedirectToPage(content=content)


class SourceAssetAddFormLookup(silvaforms.DefaultFormLookup):
    grok.context(SourceAssetAddForm)

    def fields(self):
        fields = self.form.fields
        if self.form.source is not None:
            fields += self.form.source.fields
        return fields


class SourceAssetEditForm(silvaforms.SMIEditForm):
    grok.context(ISourceAsset)

    actions = silvaforms.Actions(silvaforms.CancelEditAction())
    readOnly = False

    def update(self):
        editable = self.context.get_editable()
        if editable is not None:
            self.controller = editable.get_controller(self.request)
        else:
            self.controller = self.context.get_viewable().get_controller(
                self.request)
            self.readOnly = True

    def updateWidgets(self):
        super(SourceAssetEditForm, self).updateWidgets()
        if self.controller is not None:
            self.fieldWidgets.extend(
                self.controller.fieldWidgets(
                    ignoreRequest=self.readOnly,
                    ignoreContent=False,
                    display=self.readOnly))

    @property
    def formErrors(self):
        if self.controller is not None:
            return self.controller.formErrors
        return []

    def isEditable(self):
        return (self.controller is not None and
                not self.readOnly and
                self.controller.mode != silvaforms.DISPLAY)

    @silvaforms.action(
        _(u"Save"),
        available=isEditable,
        implements=silvaforms.IDefaultAction,
        accesskey=u'ctrl+s')
    def save(self):
        status = self.controller.save()
        if status is silvaforms.SUCCESS:
            self.send_message(_(u"Changes saved."), type="feedback")
        return status


class SourceAssetEditFormLookup(silvaforms.DefaultFormLookup):
    grok.context(SourceAssetEditForm)

    def fields(self):
        if self.form.controller is not None:
            return self.form.controller.fields
        return silvaforms.Fields()


class SourceAssetView(silvaviews.View):
    grok.context(ISourceAsset)

    def render(self):
        error = None
        try:
            # Try to fetch the controller.
            controller = self.content.get_controller(self.request)
        except SourceError, error:
            error = error
        else:
            try:
                # Try to render the source.
                return controller.render()
            except SourceError, error:
                error = error
        # Render the error if it returns something.
        message = silvaviews.render(error, self.request).strip()
        if message:
            return message
        # Return the default text used by the layout otherwise (this
        # happens with the source is published, you don't have any
        # details).
        text = _('Sorry, this ${meta_type} is not viewable.',
                 mapping={'meta_type': self.context.meta_type})
        return u'<p>%s</p>' % translate(text, context=self.request)



## Script (Python) "toc_sort_on"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=sort_on,sort_order
##title=
##
#convert the sort_on and sort_order cs params
# to the sort_order key the tocrenderingadapter
# accepts
if sort_order == "reverse":
    if sort_on == "silva":
        return "reversesilva"
    if sort_on == "alpha":
        return "reversealpha"
    elif sort_on == "chronmod":
        return "rchronmod"
return sort_on
